<?php
namespace plugins\passwordless\services;

use \IoC;
use \Validator;
use \models\User;
use plugins\passwordless\models\Magiclink;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use \Latte\Loaders\FileLoader;

/**
 * @author Jérôme Brilland
 * @version 1.0
 * @copyright (c) 2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class SendLoginLink {
    
    private $email = null;
    private $tokenValidity = null;
    
    public function __construct( string $email, string $tokenValidity ) {
        $this->email = $email;
        $this->tokenValidity = $tokenValidity;
    }
    
    public function perform() {
        $eventMediator = IoC::resolve( 'eventmediator' );
        if ( !Validator::validateNotEmpty( $this->email ) ) {
            $eventMediator->trigger( 'empty_email' );
            return;
        }
        if ( !Validator::validateEmail( $this->email ) ) {
		    $eventMediator->trigger( 'invalid_email' );
            return;
        }
        $eventMediator = IoC::resolve( 'eventmediator' );
        if ( !User::exists( ['email' => $this->email] ) ) {
            $eventMediator->trigger( 'user_not_found' );
            return;
        }
        $user = User::findByEmail( $this->email );
        $magicLink = Magiclink::findOrNew( ['user_id' => $user->user_id] );
        $magicLink->token = $this->generateToken();
        $magicLink->token_generated_at = date( 'Y-m-d H:i:s' );
        $magicLink->cookie_token = $this->generateToken();
        $magicLink->save();
        
        $this->sendEmail( $magicLink );
        
        $response = IoC::resolve( 'response' );
        $response->setCookie( 'lemoncms-magic-link-check', $magicLink->cookie_token, time() + $this->tokenValidity, '', '', false, true );
    }
    
    private function sendEmail( $magicLink ) {
        $i18n = IoC::resolve( 'i18n' );
	    $mail = new PHPMailer( true );
        $settings = IoC::resolve( 'settings' );
        
        $mail->setFrom( $settings['from_email'], $settings['from_name'] );
        $mail->addAddress( $this->email );
 
        $mail->Subject = $i18n->__( 'passwordless.your_login_link' );
        $link = BASE_URL . 'auth/login/' . $magicLink->token;
        $mail->isHTML( true );

        $latte = IoC::resolve( 'view' )->getLatte();
        $latte->getLoader()->addLoader( new \Latte\Loaders\FileLoader( APP_DIR . 'plugins' . DS . 'passwordless' . DS . TEMPLATES_DIR . 'login' ) );

        $mail->Body = $latte->renderToString( 'magic_link.html', ['link' => $link] );
        $mail->AltBody = $latte->renderToString( 'magic_link.txt', ['link' => $link] );

        IoC::resolve( 'eventmediator' )->trigger( 'before_send_email', [$mail] );
        $mail->Send();
    }
    
    private function generateToken() {
        return bin2hex( random_bytes ( 32 ) );
    }
}
